/*
 * Copyright © 2014      Jonas Ådahl
 * Copyright © 2015      Red Hat Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef POINTER_CONSTRAINTS_UNSTABLE_V1_SERVER_PROTOCOL_H
#define POINTER_CONSTRAINTS_UNSTABLE_V1_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct wl_pointer;
struct wl_region;
struct wl_surface;
struct zwp_confined_pointer_v1;
struct zwp_locked_pointer_v1;
struct zwp_pointer_constraints_v1;

extern const struct wl_interface zwp_pointer_constraints_v1_interface;
extern const struct wl_interface zwp_locked_pointer_v1_interface;
extern const struct wl_interface zwp_confined_pointer_v1_interface;

#ifndef ZWP_POINTER_CONSTRAINTS_V1_ERROR_ENUM
#define ZWP_POINTER_CONSTRAINTS_V1_ERROR_ENUM
/**
 * zwp_pointer_constraints_v1_error - wp_pointer_constraints error values
 * @ZWP_POINTER_CONSTRAINTS_V1_ERROR_ALREADY_CONSTRAINED: pointer
 *	constraint already requested on that surface
 *
 * These errors can be emitted in response to wp_pointer_constraints
 * requests.
 */
enum zwp_pointer_constraints_v1_error {
	ZWP_POINTER_CONSTRAINTS_V1_ERROR_ALREADY_CONSTRAINED = 1,
};
#endif /* ZWP_POINTER_CONSTRAINTS_V1_ERROR_ENUM */

#ifndef ZWP_POINTER_CONSTRAINTS_V1_LIFETIME_ENUM
#define ZWP_POINTER_CONSTRAINTS_V1_LIFETIME_ENUM
/**
 * zwp_pointer_constraints_v1_lifetime - the pointer constraint may
 *	reactivate
 * @ZWP_POINTER_CONSTRAINTS_V1_LIFETIME_ONESHOT: (none)
 * @ZWP_POINTER_CONSTRAINTS_V1_LIFETIME_PERSISTENT: (none)
 *
 * A persistent pointer constraint may again reactivate once it has been
 * deactivated. See the corresponding deactivation event
 * (wp_locked_pointer.unlocked and wp_confined_pointer.unconfined) for
 * details.
 */
enum zwp_pointer_constraints_v1_lifetime {
	ZWP_POINTER_CONSTRAINTS_V1_LIFETIME_ONESHOT = 1,
	ZWP_POINTER_CONSTRAINTS_V1_LIFETIME_PERSISTENT = 2,
};
#endif /* ZWP_POINTER_CONSTRAINTS_V1_LIFETIME_ENUM */

/**
 * zwp_pointer_constraints_v1 - constrain the movement of a pointer
 * @destroy: destroy the pointer constraints manager object
 * @lock_pointer: lock pointer to a position
 * @confine_pointer: confine pointer to a region
 *
 * The global interface exposing pointer constraining functionality. It
 * exposes two requests: lock_pointer for locking the pointer to its
 * position, and confine_pointer for locking the pointer to a region.
 *
 * The lock_pointer and confine_pointer requests create the objects
 * wp_locked_pointer and wp_confined_pointer respectively, and the client
 * can use these objects to interact with the lock.
 *
 * For any surface, only one lock or confinement may be active across all
 * wl_pointer objects of the same seat. If a lock or confinement is
 * requested when another lock or confinement is active or requested on the
 * same surface and with any of the wl_pointer objects of the same seat, an
 * 'already_constrained' error will be raised.
 */
struct zwp_pointer_constraints_v1_interface {
	/**
	 * destroy - destroy the pointer constraints manager object
	 *
	 * Used by the client to notify the server that it will no longer
	 * use this pointer constraints object.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
	/**
	 * lock_pointer - lock pointer to a position
	 * @id: (none)
	 * @surface: surface to lock pointer to
	 * @pointer: the pointer that should be locked
	 * @region: region of surface
	 * @lifetime: lock lifetime
	 *
	 * The lock_pointer request lets the client request to disable
	 * movements of the virtual pointer (i.e. the cursor), effectively
	 * locking the pointer to a position. This request may not take
	 * effect immediately; in the future, when the compositor deems
	 * implementation-specific constraints are satisfied, the pointer
	 * lock will be activated and the compositor sends a locked event.
	 *
	 * The protocol provides no guarantee that the constraints are ever
	 * satisfied, and does not require the compositor to send an error
	 * if the constraints cannot ever be satisfied. It is thus possible
	 * to request a lock that will never activate.
	 *
	 * There may not be another pointer constraint of any kind
	 * requested or active on the surface for any of the wl_pointer
	 * objects of the seat of the passed pointer when requesting a
	 * lock. If there is, an error will be raised. See general pointer
	 * lock documentation for more details.
	 *
	 * The intersection of the region passed with this request and the
	 * input region of the surface is used to determine where the
	 * pointer must be in order for the lock to activate. It is up to
	 * the compositor whether to warp the pointer or require some kind
	 * of user interaction for the lock to activate. If the region is
	 * null the surface input region is used.
	 *
	 * A surface may receive pointer focus without the lock being
	 * activated.
	 *
	 * The request creates a new object wp_locked_pointer which is used
	 * to interact with the lock as well as receive updates about its
	 * state. See the the description of wp_locked_pointer for further
	 * information.
	 *
	 * Note that while a pointer is locked, the wl_pointer objects of
	 * the corresponding seat will not emit any wl_pointer.motion
	 * events, but relative motion events will still be emitted via
	 * wp_relative_pointer objects of the same seat. wl_pointer.axis
	 * and wl_pointer.button events are unaffected.
	 */
	void (*lock_pointer)(struct wl_client *client,
			     struct wl_resource *resource,
			     uint32_t id,
			     struct wl_resource *surface,
			     struct wl_resource *pointer,
			     struct wl_resource *region,
			     uint32_t lifetime);
	/**
	 * confine_pointer - confine pointer to a region
	 * @id: (none)
	 * @surface: surface to lock pointer to
	 * @pointer: the pointer that should be confined
	 * @region: region of surface
	 * @lifetime: confinement lifetime
	 *
	 * The confine_pointer request lets the client request to confine
	 * the pointer cursor to a given region. This request may not take
	 * effect immediately; in the future, when the compositor deems
	 * implementation- specific constraints are satisfied, the pointer
	 * confinement will be activated and the compositor sends a
	 * confined event.
	 *
	 * The intersection of the region passed with this request and the
	 * input region of the surface is used to determine where the
	 * pointer must be in order for the confinement to activate. It is
	 * up to the compositor whether to warp the pointer or require some
	 * kind of user interaction for the confinement to activate. If the
	 * region is null the surface input region is used.
	 *
	 * The request will create a new object wp_confined_pointer which
	 * is used to interact with the confinement as well as receive
	 * updates about its state. See the the description of
	 * wp_confined_pointer for further information.
	 */
	void (*confine_pointer)(struct wl_client *client,
				struct wl_resource *resource,
				uint32_t id,
				struct wl_resource *surface,
				struct wl_resource *pointer,
				struct wl_resource *region,
				uint32_t lifetime);
};


/**
 * zwp_locked_pointer_v1 - receive relative pointer motion events
 * @destroy: destroy the locked pointer object
 * @set_cursor_position_hint: set the pointer cursor position hint
 * @set_region: set a new lock region
 *
 * The wp_locked_pointer interface represents a locked pointer state.
 *
 * While the lock of this object is active, the wl_pointer objects of the
 * associated seat will not emit any wl_pointer.motion events.
 *
 * This object will send the event 'locked' when the lock is activated.
 * Whenever the lock is activated, it is guaranteed that the locked surface
 * will already have received pointer focus and that the pointer will be
 * within the region passed to the request creating this object.
 *
 * To unlock the pointer, send the destroy request. This will also destroy
 * the wp_locked_pointer object.
 *
 * If the compositor decides to unlock the pointer the unlocked event is
 * sent. See wp_locked_pointer.unlock for details.
 *
 * When unlocking, the compositor may warp the cursor position to the set
 * cursor position hint. If it does, it will not result in any relative
 * motion events emitted via wp_relative_pointer.
 *
 * If the surface the lock was requested on is destroyed and the lock is
 * not yet activated, the wp_locked_pointer object is now defunct and must
 * be destroyed.
 */
struct zwp_locked_pointer_v1_interface {
	/**
	 * destroy - destroy the locked pointer object
	 *
	 * Destroy the locked pointer object. If applicable, the
	 * compositor will unlock the pointer.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
	/**
	 * set_cursor_position_hint - set the pointer cursor position
	 *	hint
	 * @surface_x: surface-local x coordinate
	 * @surface_y: surface-local y coordinate
	 *
	 * Set the cursor position hint relative to the top left corner
	 * of the surface.
	 *
	 * If the client is drawing its own cursor, it should update the
	 * position hint to the position of its own cursor. A compositor
	 * may use this information to warp the pointer upon unlock in
	 * order to avoid pointer jumps.
	 *
	 * The cursor position hint is double buffered. The new hint will
	 * only take effect when the associated surface gets it pending
	 * state applied. See wl_surface.commit for details.
	 */
	void (*set_cursor_position_hint)(struct wl_client *client,
					 struct wl_resource *resource,
					 wl_fixed_t surface_x,
					 wl_fixed_t surface_y);
	/**
	 * set_region - set a new lock region
	 * @region: region of surface
	 *
	 * Set a new region used to lock the pointer.
	 *
	 * The new lock region is double-buffered. The new lock region will
	 * only take effect when the associated surface gets its pending
	 * state applied. See wl_surface.commit for details.
	 *
	 * For details about the lock region, see wp_locked_pointer.
	 */
	void (*set_region)(struct wl_client *client,
			   struct wl_resource *resource,
			   struct wl_resource *region);
};

#define ZWP_LOCKED_POINTER_V1_LOCKED	0
#define ZWP_LOCKED_POINTER_V1_UNLOCKED	1

#define ZWP_LOCKED_POINTER_V1_LOCKED_SINCE_VERSION	1
#define ZWP_LOCKED_POINTER_V1_UNLOCKED_SINCE_VERSION	1

static inline void
zwp_locked_pointer_v1_send_locked(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, ZWP_LOCKED_POINTER_V1_LOCKED);
}

static inline void
zwp_locked_pointer_v1_send_unlocked(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, ZWP_LOCKED_POINTER_V1_UNLOCKED);
}

/**
 * zwp_confined_pointer_v1 - confined pointer object
 * @destroy: destroy the confined pointer object
 * @set_region: set a new confine region
 *
 * The wp_confined_pointer interface represents a confined pointer state.
 *
 * This object will send the event 'confined' when the confinement is
 * activated. Whenever the confinement is activated, it is guaranteed that
 * the surface the pointer is confined to will already have received
 * pointer focus and that the pointer will be within the region passed to
 * the request creating this object. It is up to the compositor to decide
 * whether this requires some user interaction and if the pointer will warp
 * to within the passed region if outside.
 *
 * To unconfine the pointer, send the destroy request. This will also
 * destroy the wp_confined_pointer object.
 *
 * If the compositor decides to unconfine the pointer the unconfined event
 * is sent. The wp_confined_pointer object is at this point defunct and
 * should be destroyed.
 */
struct zwp_confined_pointer_v1_interface {
	/**
	 * destroy - destroy the confined pointer object
	 *
	 * Destroy the confined pointer object. If applicable, the
	 * compositor will unconfine the pointer.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
	/**
	 * set_region - set a new confine region
	 * @region: region of surface
	 *
	 * Set a new region used to confine the pointer.
	 *
	 * The new confine region is double-buffered. The new confine
	 * region will only take effect when the associated surface gets
	 * its pending state applied. See wl_surface.commit for details.
	 *
	 * If the confinement is active when the new confinement region is
	 * applied and the pointer ends up outside of newly applied region,
	 * the pointer may warped to a position within the new confinement
	 * region. If warped, a wl_pointer.motion event will be emitted,
	 * but no wp_relative_pointer.relative_motion event.
	 *
	 * The compositor may also, instead of using the new region,
	 * unconfine the pointer.
	 *
	 * For details about the confine region, see wp_confined_pointer.
	 */
	void (*set_region)(struct wl_client *client,
			   struct wl_resource *resource,
			   struct wl_resource *region);
};

#define ZWP_CONFINED_POINTER_V1_CONFINED	0
#define ZWP_CONFINED_POINTER_V1_UNCONFINED	1

#define ZWP_CONFINED_POINTER_V1_CONFINED_SINCE_VERSION	1
#define ZWP_CONFINED_POINTER_V1_UNCONFINED_SINCE_VERSION	1

static inline void
zwp_confined_pointer_v1_send_confined(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, ZWP_CONFINED_POINTER_V1_CONFINED);
}

static inline void
zwp_confined_pointer_v1_send_unconfined(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, ZWP_CONFINED_POINTER_V1_UNCONFINED);
}

#ifdef  __cplusplus
}
#endif

#endif
